package com.icontrols.dust;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestTemplate;

import org.json.simple.JSONObject; 
import org.json.simple.parser.JSONParser;


import com.fasterxml.jackson.databind.ObjectMapper;

public class CenterDataRequest {
	
	private static final Logger logger = LoggerFactory.getLogger(CenterDataRequest.class);
	
	public CenterDataRequest()
	{

		getSiteCode();
		ThreadRequestHandler handler = new ThreadRequestHandler();
        handler.start(); // start() --> run() 호출
        logger.debug("[CenterDataRequest] End");
	}
	
	 public boolean isWindows()
	 {
		    String OS = System.getProperty("os.name").toLowerCase();
		    
		    if( OS.indexOf("win") >= 0 )
		    {
		    	logger.info("운영체제 - 윈도우");
		    	return true;
		    }
		    logger.info("운영체제 - 리눅스");
		    return false;
	 }
	 
	 public String getSiteCode()
	 {
		    String  checkFileName;
		    if( isWindows() )
		    {
		    	checkFileName = DEF.site_code_file;
		    }
		    else 
		    {
		    	checkFileName = DEF.site_code_file;
		    }
		    
			File checkFile = new File(checkFileName);
			
			logger.info("getSiteCode checkFile.exists() : " + checkFile.exists() );
			if (checkFile.exists())
			{
				try
				{
					BufferedReader reader = new BufferedReader(	new FileReader(checkFileName));
					String str;
					
					while ((str = reader.readLine()) != null) {
						logger.info("getSiteCode : " + str);
						DEF.site_code_value = str;
					}
				}catch(Exception e )
				{
					
				}

				return DEF.site_code_value;
			}
			else
				return null;
	 }
}

class ThreadRequestHandler extends Thread {
	 private static final Logger logger = LoggerFactory.getLogger(ThreadRequestHandler.class);
   
	 //생성자
	 public ThreadRequestHandler() {
			reqeustCentenDustData();
	 }

	 //start() 메소드 호출 시 실행됩니다.
	 public void run() {
		 
		 logger.debug("[ThreadRequestHandler] Start");
			while(true)
			{

				try
				{
					Thread.sleep(1000); // 30초마다 라이브 체크 하도록 한다.
					
					Calendar cal = new GregorianCalendar();
		    		if(cal.get(Calendar.MINUTE) == 9 &&  ( cal.get(Calendar.SECOND) == 0 || cal.get(Calendar.SECOND) == 30 ))
		    		{
		    			// 매시간 정보를 갱신하도록 한다.
		    			reqeustCentenDustData();
		    		}
				}
				catch(Exception e)
				{
					
				}
			}
		//logger.debug("[ThreadRequestHandler] End");
	 }
	 
	 public void reqeustCentenDustData()
	 {
			DEF.data.period = "HOUR_10_15";
			DEF.data.khaiGrade = "1";
			DEF.data.pm10Grade1h = "2";
			DEF.data.pm25Grade1h = "3";
			DEF.data.dustValue = "1";
			
			//logger.debug("[reqeustCentenDustData] DEF.data.dustValue : " + DEF.data.dustValue);
			
			try
			{
				GetRestTemplate rt = new GetRestTemplate();
				RestTemplate restTemplate = rt.get2Modify();
				
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
	
				String url = DEF.dustCenterURL + ":" + DEF.dustCenterPort + "/dustinfo";
				
				String answer = "";

				Date now = new Date();
				String ServerTime = new SimpleDateFormat("yyyy.MM.dd_HH.mm.ss" ).format(now);
				
				Map<String, Object> retMap = new HashMap<>();

				retMap.put("site_code", DEF.site_code_value);
				retMap.put("req_time", ServerTime);
								
				//logger.debug("SendStatusResult retMap value : " +  retMap.toString());
				
				ObjectMapper mapper = new ObjectMapper(); 
				String json = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(retMap); // pretty print 
				logger.info("\r\n"+json + "\r\n");
				
				//logger.debug("SendStatusResult retMap value : " +  retMap.toString());
				
				HttpEntity<Map<String, Object>> entity = new HttpEntity<Map<String, Object>>(retMap, headers);
				answer = restTemplate.postForObject(url, entity, String.class);
				logger.debug("SendStatusResult answer :" + answer);
				// 응답도 다시 파싱하여 재전송 하거나 해야 한다.
				JSONParser parser = new JSONParser(); 
				Object obj = parser.parse( answer ); 
				JSONObject jsonObj = (JSONObject) obj;
				
				Map<String, Object> objresult = (Map<String, Object>)jsonObj.get("report");
				logger.debug("answer result :" + objresult.get("result"));
				
				Map<String, Object> objreport = (Map<String, Object>)jsonObj.get("dust_info");
				DEF.data.version = (String)objreport.get("version");
				logger.debug("answer version :" + DEF.data.version);
				
				DEF.data.khaiGrade = (String)objreport.get("khaiGrade");
				logger.debug("answer khaiGrade :" + DEF.data.khaiGrade);

				DEF.data.pm10Grade1h = (String)objreport.get("pm10Grade1h");
				logger.debug("answer pm10Grade1h :" + DEF.data.pm10Grade1h);
				
				DEF.data.pm25Grade1h = (String)objreport.get("pm25Grade1h");
				logger.debug("answer pm25Grade1h :" + DEF.data.pm25Grade1h);
				
				DEF.data.dustValue = (String)objreport.get("dustValue");
				logger.debug("answer dustValue :" + DEF.data.dustValue);
				
				Map<String, Object> objsetting = (Map<String, Object>)jsonObj.get("setting");
				DEF.data.period = (String)objsetting.get("period");
				logger.debug("answer period :" + DEF.data.period);
				
			}
			catch( Exception e)
			{
				logger.debug("SendStatusResult Exception e :" + e.toString());
			}
	}
}

